@@ -57,6 +57,8 @@ public:
     QHash<QString, KQuickAddons::ConfigModule *> kcms;
 
     void appendCategory(ConfigCategory *c);
+    void removeCategory(ConfigCategory *c);
+    void removeCategoryAt(int index);
     void clear();
     QVariant get(int row) const;
 
@@ -133,6 +135,10 @@ void ConfigModelPrivate::clear()
 
 void ConfigModelPrivate::appendCategory(ConfigCategory *c)
 {
+    if (!c) {
+        return;
+    }
+
     q->beginInsertRows(QModelIndex(), categories.size(), categories.size());
     categories.append(c);
 
@@ -154,6 +160,31 @@ void ConfigModelPrivate::appendCategory(ConfigCategory *c)
     emit q->countChanged();
 }
 
+void ConfigModelPrivate::removeCategory(ConfigCategory *c)
+{
+    const int index = categories.indexOf(c);
+    if (index > -1) {
+        removeCategoryAt(index);
+    }
+}
+
+void ConfigModelPrivate::removeCategoryAt(int index)
+{
+    if (index < 0 || index >= categories.count()) {
+        return;
+    }
+
+    q->beginRemoveRows(QModelIndex(), index, index);
+
+    ConfigCategory *c = categories.takeAt(index);
+    if (c->parent() == q) {
+        c->deleteLater();
+    }
+
+    q->endRemoveRows();
+    emit q->countChanged();
+}
+
 QVariant ConfigModelPrivate::get(int row) const
 {
     QVariantMap value;
@@ -175,15 +206,7 @@ ConfigModel::ConfigModel(QObject *parent)
     : QAbstractListModel(parent),
       d(new ConfigModelPrivate(this))
 {
-    QHash<int, QByteArray> roleNames;
-    roleNames[NameRole] = "name";
-    roleNames[IconRole] = "icon";
-    roleNames[SourceRole] = "source";
-    roleNames[PluginNameRole] = "pluginName";
-    roleNames[VisibleRole] = "visible";
-    roleNames[KCMRole] = "kcm";
 
-    setRoleNames(roleNames);
 }
 
 ConfigModel::~ConfigModel()
@@ -255,6 +278,18 @@ QVariant ConfigModel::data(const QModelIndex &index, int role) const
     }
 }
 
+QHash<int, QByteArray> ConfigModel::roleNames() const
+{
+    return {
+        {NameRole, "name"},
+        {IconRole, "icon"},
+        {SourceRole, "source"},
+        {PluginNameRole, "pluginName"},
+        {VisibleRole, "visible"},
+        {KCMRole, "kcm"},
+    };
+}
+
 QVariant ConfigModel::get(int row) const
 {
     return d->get(row);
@@ -283,6 +318,21 @@ void ConfigModel::appendCategory(const QString &iconName, const QString &name,
     d->appendCategory(cat);
 }
 
+void ConfigModel::appendCategory(ConfigCategory *category)
+{
+    d->appendCategory(category);
+}
+
+void ConfigModel::removeCategory(ConfigCategory *category)
+{
+    d->removeCategory(category);
+}
+
+void ConfigModel::removeCategoryAt(int index)
+{
+    d->removeCategoryAt(index);
+}
+
 void ConfigModel::clear()
 {
     d->clear();
